@@ -33,7 +33,6 @@ public:
   void fix_length_and_dec();
   enum_field_types field_type() const  { return MYSQL_TYPE_GEOMETRY; }
   Field *tmp_table_field(TABLE *t_arg);
-  virtual int get_geometry_type() const;
   bool is_null() { (void) val_int(); return null_value; }
 };
 
@@ -92,7 +91,7 @@ public:
   Item_func_centroid(Item *a): Item_geometry_func(a) {}
   const char *func_name() const { return "centroid"; }
   String *val_str(String *);
-  int get_geometry_type() const;
+  Field::geometry_type get_geometry_type() const;
 };
 
 class Item_func_envelope: public Item_geometry_func
@@ -101,7 +100,7 @@ public:
   Item_func_envelope(Item *a): Item_geometry_func(a) {}
   const char *func_name() const { return "envelope"; }
   String *val_str(String *);
-  int get_geometry_type() const;
+  Field::geometry_type get_geometry_type() const;
 };
 
 class Item_func_point: public Item_geometry_func
@@ -111,7 +110,7 @@ public:
   Item_func_point(Item *a, Item *b, Item *srid): Item_geometry_func(a, b, srid) {}
   const char *func_name() const { return "point"; }
   String *val_str(String *);
-  int get_geometry_type() const;
+  Field::geometry_type get_geometry_type() const;
 };
 
 class Item_func_spatial_decomp: public Item_geometry_func
@@ -225,8 +224,13 @@ public:
       DBUG_ASSERT(0);  // Should never happened
       return "sp_unknown"; 
     }
-    }
-  void print(String *str) { Item_func::print(str); }
+  }
+
+  virtual inline void print(String *str, enum_query_type query_type)
+  {
+    Item_func::print(str, query_type);
+  }
+
   void fix_length_and_dec() { maybe_null= 1; }
   bool is_null() { (void) val_int(); return null_value; }
 };
@@ -374,11 +378,11 @@ public:
   void fix_length_and_dec() { max_length= 10; maybe_null= 1; }
 };
 
-#define GEOM_NEW(obj_constructor) new obj_constructor
+#define GEOM_NEW(thd, obj_constructor) new (thd->mem_root) obj_constructor
 
 #else /*HAVE_SPATIAL*/
 
-#define GEOM_NEW(obj_constructor) NULL
+#define GEOM_NEW(thd, obj_constructor) NULL
 
 #endif
 
