@@ -37,7 +37,7 @@
 MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking)
 {
   int save_errno,errpos=0;
-  uint files= 0, i, dir_length, length, key_parts, min_keys= 0;
+  uint files= 0, i, dir_length, length, UNINIT_VAR(key_parts), min_keys= 0;
   ulonglong file_offset=0;
   char name_buff[FN_REFLEN*2],buff[FN_REFLEN],*end;
   MYRG_INFO *m_info=0;
@@ -49,8 +49,6 @@ MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking)
   my_bool bad_children= FALSE;
   DBUG_ENTER("myrg_open");
 
-  LINT_INIT(key_parts);
-
   bzero((char*) &file,sizeof(file));
   if ((fd=my_open(fn_format(name_buff,name,"",MYRG_NAME_EXT,
                             MY_UNPACK_FILENAME|MY_APPEND_EXT),
@@ -394,7 +392,7 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
   int        save_errno;
   uint       idx;
   uint       child_nr;
-  uint       key_parts;
+  uint       UNINIT_VAR(key_parts);
   uint       min_keys;
   my_bool    bad_children= FALSE;
   DBUG_ENTER("myrg_attach_children");
@@ -411,7 +409,6 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
   rc= 1;
   errpos= 0;
   file_offset= 0;
-  LINT_INIT(key_parts);
   min_keys= 0;
   child_nr= 0;
   while ((myisam= (*callback)(callback_param)))
